<?php

namespace Drupal\custom_data\Controller;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\Controller\EntityViewController;
use Drupal\custom_data\CustomDataInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class CustomDataController.
 */
class CustomDataController implements ContainerInjectionInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->renderer = $container->get('renderer');
    return $instance;
  }

  /**
   * Provides a page to render a single custom data item.
   *
   * This wraps the default entity view controller in the way that a check is
   * performed, whether the according type is configured to have a canonical
   * URL. It passes the rendering to the default view controller if enabled
   * to do so, and otherwise throws an exception to show a 404 page instead.
   *
   * @param \Drupal\custom_data\CustomDataInterface $custom_data
   *   The entity to render.
   * @param string $view_mode
   *   (optional) The view mode that should be used to display the entity.
   *   Defaults to 'full'.
   *
   * @return array
   *   A render array as expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   When the custom type is not configured to have canonical URLs.
   */
  public function view(CustomDataInterface $custom_data, $view_mode = 'full') {
    /** @var \Drupal\custom_data\CustomDataTypeInterface $data_type */
    $data_type = $this->entityTypeManager->getStorage('custom_data_type')->load($custom_data->bundle());
    if (!$data_type->hasCanonical()) {
      throw new NotFoundHttpException();
    }
    return (new EntityViewController($this->entityTypeManager, $this->renderer))->view($custom_data, $view_mode);
  }

}
